import sys
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem
from OFS.Cache import Cacheable
from OFS.PropertyManager import PropertyManager
from OFS.ObjectManager import ObjectManager
from Globals import InitializeClass
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from Products.PageTemplates.GlobalTranslationService import getGlobalTranslationService
from Settings import InterfaceMetaType, i18ndomain
from Permissions import RakunView
from RequestManagement import RequestManager
from RakunUtils import translate, get_i18n_language
from IObjects import IRpc

manage_addInterfaceForm = PageTemplateFile('zmi/addInterface', globals(),
                                           __name__="manage_addInterfaceForm")

default_text = '''
<tal:block define="absolute_url template/absolute_url;
                   interface    nocall:template;">
    <metal:block use-macro="here/interface_template/macros/master" />
</tal:block>
'''

class Module(RequestManager):
    """ 
        Rakun Module
    """
    
    # Private Methods
    def _getObject(self,object_name):
        if object_name in self.objectIds():
            return getattr(self,object_name,None)
        else:
            return None
        
    def _getScript(self,script_name):
        script = self._getObject(script_name)
        if script:
            if not IRpc.isImplementedBy(script):
                return script
        return None
    
    def _execMethod(self,method_name):
        method = self._getScript(method_name)
        if method:
            return method()        
        
    def _logException(self):
        try: self.error_log.raising(sys.exc_info())
        except: pass


    def index_html(self,REQUEST):
        """  Default view method
        """    
        return self(REQUEST)
    
    def getTitle(self):
        title = self._execMethod("Title")
        if not title:
            title = self.title_or_id()
        return title     
    
    def get_i18n_domain(self):
        """Get i18n domain, if any.
        """
        i18n_domain = getattr(self, 'i18n_domain',i18ndomain)
        if not i18n_domain:
            i18n_domain = i18ndomain
        return i18n_domain
    
    def get_i18n_message(self,message):
        """
        """
        module_i18n_domain = self.get_i18n_domain()        
        if module_i18n_domain:
            i18n_message =  translate(module_i18n_domain,message,
                                      context=self,default=message,as_unicode=1)
        else:
            i18n_message = message
            
        if i18n_message == message and i18ndomain != module_i18n_domain:
            i18n_message =  translate(i18ndomain, message, context=self,
                                      default=message,as_unicode=1)
        return i18n_message.encode(self.get_form_encoding())
    
    def defaultCondition(self, REQUEST=None):
        """run default_condition script """
        if REQUEST is not None:
            self._setContext(**REQUEST)
        return self._execMethod('default_condition')
        
    def rpcDefaultCondition(self, REQUEST=None):
        """defaultCondition rpc interface"""
        condition = self.defaultCondition(REQUEST)
        if condition:
            return condition.toRpc()
        else:
            return ''
    
   

class Interface(Module, ObjectManager, PropertyManager, Cacheable, ZopePageTemplate):
    """
        Rakun Interface
    """
    meta_type = InterfaceMetaType
    
    _text = default_text
    
    om_icons = None
    master_entity_view = 'form'
    master_entity = ""
    i18n_domain = "rakun"
    form_encoding = "UTF-8"
    relations = []
    detail_entities = []
    
    _properties = ({'id':'title','type':'string','mode':'rw'}
                   ,{'id':'master_entity','type':'string','mode':'r'} 
                   ,{'id':'master_entity_view','type':'string','mode':'r'}
                   ,{'id':'i18n_domain','type':'string','mode':'rw'}    
                   ,{'id':'form_encoding','type':'string','mode':'rw'}    
                   )
    
    manage_options = (PropertyManager.manage_options
                      +({'label':'Settings','action':'manage_Settings'},
                        {'label':'Edit', 'action':'pt_editForm'},
                        {'label':'View','action':'index_html'},)
                      +ObjectManager.manage_options
                      +Cacheable.manage_options
                      )
    
    manage_Settings = PageTemplateFile('zmi/interfaceSettings', globals(),
                                       __name__="manage_Settings")
    
    security = ClassSecurityInfo()
    security.declareObjectProtected(RakunView)
 
    def __init__(self,id,title=""):
        "Initialize Interface Object"
        self.id = id
        self.title = title


    def get_form_encoding(self):
        return self.form_encoding
    
    
    def getAttributes(self, REQUEST=None):
        """ """
        if REQUEST:
            self._setContext(**REQUEST)
        return {'title': self.get_i18n_message( self.getTitle() ) ,
                'applicationurl':self.getApplicationUrl(),
                'master_entity_path':self.getMasterEntityPath(),
                'master_entity_view':self.master_entity_view,
                'master_relations':self.getMasterRelations()}
    
    
    def getApplicationUrl(self):
        """ """
        try: return self.rakun_applications.absolute_url()
        except: return self.portal_url()
        
        
    def getMasterEntityPath(self):
        """ Returns master entity path
        """
        return self.master_entity
    
    
    def isRelationSelected(self,relation):
        """ Checks that the relation is selected
        """
        for r in self.relations:
            if (relation['detail_entity'] == r['detail_entity'] and \
                relation['foreign_field'] == r['foreign_field']) or \
               relation['detail_entity'] in self.detail_entities: # for backward compability
                return 1
        return 0
    
    def isSelectedView(self, relation, view):
        """ """
        for r in self.relations:
            if(relation['detail_entity'] == r['detail_entity'] and \
               relation['foreign_field'] == r['foreign_field'] and \
               r.get('view') == view):            
                return True
        return False
              
    def getSelectedView(self, relation):
        """ """
        for r in self.relations:
            if(relation['detail_entity'] == r['detail_entity'] and \
               relation['foreign_field'] == r['foreign_field']):
                return r.get('view')
        return None
            
            
    def getMasterRelations(self):
        """ Returns master relations 
        """
        relations = []
        for relation in self.relation_search(self.master_entity):
            if self.isRelationSelected(relation):
                relation['view'] = self.getSelectedView(relation)
                relations.append(relation)                
        return relations
    
    security.declareProtected(RakunView,"getStyleObjects")
    def getStyleObjects(self):
        """ Returns interface style objects
        """
        styles = []
        style = self._getObject("custom.css")
        if style: styles.append(style)
        master_path = self.getMasterEntityPath()
        paths = [relation['detail_entity'] for relation in self.getMasterRelations() ]
        paths.insert(0,master_path)
        for path in paths:
            entity = eval("self."+path.replace("/","."))
            style = entity.getStyleObject()
            if style: styles.append(style)
        return styles
    
    security.declareProtected(RakunView,"getJScriptObjects")
    def getJScriptObjects(self):
        """ Returns interface jscript objects
        """
        jscripts = []
        jscript = self._getObject("custom.js")
        if jscript: jscripts.append(jscript)
        master_path = self.getMasterEntityPath()
        paths = [relation['detail_entity'] for relation in self.getMasterRelations() ]
        paths.insert(0,master_path)
        for path in paths:
            entity = eval("self."+path.replace("/","."))
            jscript = entity.getJScriptObject()
            if jscript: jscripts.append(jscript)
        return jscripts
   
    security.declareProtected('View management screens', 'manage_changeMasterEntity')
    def manage_changeMasterEntity(self,master_entity,master_entity_view, REQUEST=None):
        """ change Master Entity
        """
        if self.master_entity != master_entity:
            self.relations = []
        self.master_entity = master_entity
        self.master_entity_view = master_entity_view
        if REQUEST:
            message="Master entity changed."
            return self.manage_Settings(self,REQUEST,
                                        manage_tabs_message=message)
        
    security.declareProtected('View management screens', 'manage_saveDetailEntities')
    def manage_saveRelations(self,relations=[],REQUEST=None):
        """ change Detail Entities
        """
        self.detail_entities = [] # for backward compability
        self.relations = []
        for relation in relations:
            if relation.has_key('selected'):
                self.relations.append(relation)
        if REQUEST:
            message="Detail entities changed."
            return self.manage_Settings(self,REQUEST,
                                        manage_tabs_message=message)
    


InitializeClass(Interface)        
    
def manage_addInterface(dispatcher,id,title="",REQUEST=None):
    """add new Interface """
    q = Interface(id,title)
    dispatcher.Destination()._setObject(id,q)
    q._text = default_text
    if REQUEST is not None:
        url = dispatcher.DestinationURL() + '/' + id + '/manage_propertiesForm'
        REQUEST['RESPONSE'].redirect(url)